from django.shortcuts import render, redirect
from django.contrib import messages
from django.http import HttpResponse
from branch.models import CompanyBranch
from quotations.models import Inquiry, QualityFeedback, Quotation
from warranty.models import Warranty

def add_warranty(request):
    if request.method == "POST":
        # Retrieve form data
        description = request.POST.get('description')
        quotation_id = request.POST.get('quotation_id')

        # Retrieve the PDF file from the request
        warranty_pdf = request.FILES.get('warranty_pdf')

        # Print the inputs for debugging
        print(f"Description: {description}")
        print(f"Quotation ID: {quotation_id}")
        print(f"Warranty PDF: {warranty_pdf}")

        # Check if the file, description, and quotation_id are provided
        if not warranty_pdf or not description or not quotation_id:
            messages.error(request, "All fields are required.")
            return redirect('sales:add_warranty')  # Redirect back to the warranty form page
        
        try:
            # Get the Quotation object using quotation_id
            quotation = Quotation.objects.get(id=quotation_id)
        except Quotation.DoesNotExist:
            messages.error(request, "Quotation not found.")
            return redirect('sales:add_warranty')  # Redirect if the quotation is not found
        
        # Check if a warranty already exists for the quotation
        existing_warranty = Warranty.objects.filter(quotation=quotation).first()

        if existing_warranty:
            # Update the existing warranty
            existing_warranty.description = description
            existing_warranty.warranty_pdf = warranty_pdf
            existing_warranty.save()
            quotation.has_warranty = True
            quotation.save()

            # Display a success message for updating the warranty
            messages.success(request, "Warranty successfully updated.")
        else:
            # Create a new warranty
            warranty = Warranty(
                quotation=quotation,  # Linking the warranty to the quotation
                description=description,
                warranty_pdf=warranty_pdf   
            )
            quotation.has_warranty = True
            quotation.save()
            warranty.save()

            # Display a success message for adding the warranty
            messages.success(request, "Warranty successfully added.")

        # Redirect to the invoices or another page of your choice
        return redirect('sales:invoices')

    # If the request method is not POST, simply redirect back to the invoice page
    return redirect('sales:invoices')



from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden 

@login_required
def quality_feedback(request):
    if not (request.user.is_admin_user or request.user.is_super_user or request.user.is_sales):
        return HttpResponseForbidden("You are not authorized to view this page.")

    # Handle POST request to save quality feedback
    if request.method == 'POST':
        quotation_id = request.POST.get('quotation_id')
        print(quotation_id)
        quotation = get_object_or_404(Quotation, id=quotation_id)

        # Get feedback details from the request
        customer_comment = request.POST.get('customer_comment')
        action_taken = request.POST.get('action_taken')

        if not customer_comment or not action_taken:
            messages.error(request, "Both customer comment and action taken are required.") 

        # Save feedback
        QualityFeedback.objects.create(
            quotation=quotation,
            customer_comment=customer_comment,
            action_taken=action_taken,
        )

        # Add success message
        messages.success(request, "Quality feedback successfully saved.")

        # Redirect back to quotations page
        return redirect('sales:invoices')

    # Add error message for invalid method
    messages.error(request, "Invalid request method.")
    return redirect('sales:invoices')


from django.shortcuts import render, get_object_or_404, redirect
from django.contrib import messages  
from django.http import JsonResponse
from django.core.files.storage import FileSystemStorage 
from django.core.exceptions import ValidationError
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.core.files.storage import FileSystemStorage
from math import floor


def outsource_inquiry(request):
    if request.method == 'POST':
        inquiry_id = request.POST.get('inquiryId')
        outsourced_company = request.POST.get('outsourcedCompany')
        invoice_document = request.FILES.get('invoiceDocument')

        # Retrieve the Inquiry object by ID or return 404 if not found
        inquiry = get_object_or_404(Inquiry, id=inquiry_id)

        # Update the Inquiry object with the outsourced company
        inquiry.outsourced_company = outsourced_company
        inquiry.production_status = 'Outsourced'  # Set the status to 'Outsourced'

        # Handle the file upload if present
        if invoice_document:
            fs = FileSystemStorage()
            filename = fs.save(invoice_document.name, invoice_document)
            inquiry.invoice_document = fs.url(filename)

        # Save the updated Inquiry object
        inquiry.save()

        # Add a success message
        messages.success(request, 'Outsource details saved successfully!')

        # Check if all inquiries in the quotation are either 'Outsourced' or 'Produced'
        all_outsourced_or_produced = Inquiry.objects.filter(quotation=inquiry.quotation).exclude(
            production_status__in=['Outsourced', 'Produced']
        ).count() == 0

        # If all inquiries are outsourced or produced, trigger the dispatch action
        if all_outsourced_or_produced:
            quotation = inquiry.quotation
            quotation.status = 'Produced'  
            quotation.save()

            # Add a message indicating that the quotation is now ready for dispatch
            messages.success(request, 'All inquiries have been outsourced or produced. The quotation is now ready for dispatch.')

        # Redirect to the quotation production page with the inquiry's quotation_id
        return redirect('production:quotation_production', quotation_id=inquiry.quotation.id)

    # If the method is not POST, add an error message
    messages.error(request, 'Invalid request method')
    return JsonResponse({'message': 'Invalid request method', 'status': 'error'}, status=400)


from rest_framework.views import APIView
from rest_framework.response import Response
from django.db.models.functions import TruncMonth
from django.db.models import Sum 
from datetime import datetime
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.db.models import Sum
from django.db.models.functions import TruncMonth
from datetime import datetime  

from datetime import datetime
from calendar import month_name
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.db.models.functions import TruncMonth
from django.db.models import Sum 
from datetime import datetime
from calendar import month_name
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.db.models.functions import TruncMonth
from django.db.models import Sum 

from datetime import datetime
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.db.models import Sum
from django.db.models.functions import TruncMonth 

class SalesByBranchAPIView(APIView):
    def get(self, request):
        # Extract the year and payment_status from query parameters
        year = request.query_params.get('year', datetime.now().year)
        payment_status = request.query_params.get('payment_status', 'ALL')

        try:
            year = int(year)  # Ensure the year is an integer
        except ValueError:
            return Response({"error": "Invalid year provided."}, status=status.HTTP_400_BAD_REQUEST)

        if payment_status not in ['ALL', 'PAID', 'UNPAID', 'PARTIAL PAID']:
            return Response({"error": "Invalid payment status provided."}, status=status.HTTP_400_BAD_REQUEST)

        data = {
            "branches": []
        }

        # Fetch all company branches
        branches = CompanyBranch.objects.all()

        # Iterate through all branches
        for branch in branches:
            # Filter quotations by branch's sellers, the specified year, and the payment status
            filters = {
                'seller__branch': branch,
                'created_at__year': year
            }

            if payment_status != 'ALL':  # Only apply the filter if it's not 'ALL'
                filters['payment_status'] = payment_status

            quotations = Quotation.objects.filter(**filters)

            # Group and sum sales by month
            monthly_sales = (
                quotations
                .annotate(month=TruncMonth('created_at'))  # Truncate date to month
                .values('month')  # Group by month
                .annotate(total_sales=Sum('total_amount'))  # Sum total_amount per month
                .order_by('month')  # Order by month
            )

            # Create a dictionary for all months initialized with 0 sales
            all_months_sales = {month_name[i]: 0 for i in range(1, 13)}

            # Update the dictionary with actual sales data
            for sale in monthly_sales:
                month_name_str = sale['month'].strftime("%B")  # Convert to month name
                all_months_sales[month_name_str] = sale['total_sales'] or 0

            # Convert the dictionary to a list of {month, value} format
            monthly_sales_data = [
                {"month": month, "value": value}
                for month, value in all_months_sales.items()
            ]

            # Add branch sales data to the response
            data["branches"].append({
                "company_name": branch.branch_name,
                "monthly_sales": monthly_sales_data
            })

        # Return the response in a pretty nested JSON format
        return Response(data, status=status.HTTP_200_OK)
